Spine.Mqtt = (function() {
	"use strict";

	var webmqtt;
	var connected = false;
	var params = Spine.getMqttConnectParams();

	function init() {
		if (Spine.Debugging === true)
			console.log('Spine.Mqtt.init');
		webmqtt = new Paho.MQTT.Client(params.host, params.port, params.clientId);
		webmqtt.onMessageArrived = onMessageArrived;
		webmqtt.onConnectionLost = onConnectionLost;
		webmqtt.onMessageDelivered = onMessageDelivered;
		$('#webmqttmessage').css('color', 'black');
		$('#webmqttmessage').text('Not connected');

		function onMessageArrived(message) {
			if (Spine.Debugging === true)
				console.log('Spine.Mqtt.onMessageArrived: ' + message.destinationName + ' - ' + message.payloadString);
			if (Spine.Content.onMessageArrived !== 'undefined' && $.isFunction(Spine.Content.onMessageArrived))
				Spine.Content.onMessageArrived(message.destinationName, message.payloadString);
		};

		function onMessageDelivered(message) {
			if (Spine.Debugging === true)
				console.log('Spine.Mqtt.onMessageDelivered: ' + message.destinationName + ' - ' + message.payloadString);
			if (Spine.Content.onMessageDelivered !== 'undefined' && $.isFunction(Spine.Content.onMessageDelivered))
				Spine.Content.onMessageDelivered(message.destinationName, message.payloadString);
		};

		function onConnectionLost(responseObject) {
			connected = false;
			$('#webmqttmessage').css('color', 'red');
			if (responseObject.errorCode !== 0) {
				if (Spine.Debugging === true)
					console.log('Spine.Mqtt.onConnectionLost: ' + responseObject.errorMessage);
				$('#webmqttmessage').text('Connection Lost');
				setTimeout(connect, 1000);
			}
			else
				$('#webmqttmessage').text('Disconnected');
		};
	};

	function connect() {
		if (connected === true)
			return;
		if (Spine.Debugging === true)
			console.log('Spine.Mqtt.connect');
		setTimeout(connect, 10000);
		webmqtt.connect({onSuccess: onConnect, cleanSession: params.cleanSession, useSSL: params.useSSL, mqttVersion: 3});

		function onConnect() {
			connected = true;
			$('#webmqttmessage').css('color', 'green');
			$('#webmqttmessage').text('Connected');
			if (Spine.Debugging === true)
				console.log('Spine.Mqtt.onConnect');
			if (Spine.Content.onConnect !== 'undefined' && $.isFunction(Spine.Content.onConnect))
				Spine.Content.onConnect();
		};
	};

	function disconnect() {
		if (connected === true) {
			webmqtt.disconnect();
			if (Spine.Debugging === true)
				console.log('Spine.Mqtt.disconnect');
		}
	};

	function subscribe(topic, options) {
		if (connected === true) {
			webmqtt.subscribe(topic, options);
			if (Spine.Debugging === true)
				console.log('Spine.Mqtt.subscribe');
		}
	};

	function unsubscribe(topic) {
		if (connected === true) {
			webmqtt.unsubscribe(topic);
			if (Spine.Debugging === true)
				console.log('Spine.Mqtt.unsubscribe');
		}
	};

	function publish(topic, payload, retained) {
		if (connected === true) {
			var message = new Paho.MQTT.Message(payload);
			message.destinationName = topic;
			message.qos = 1;
			message.retained = retained ? true : false;
			webmqtt.send(message);
			if (Spine.Debugging === true)
				console.log('Spine.Mqtt.publish');
		}
	};

	function isConnected() {
		return connected;
	};

	return {
		init:        init,
		connect:     connect,
		disconnect:  disconnect,
		subscribe:   subscribe,
		unsubscribe: unsubscribe,
		publish:     publish,
		isConnected: isConnected
	};
})();
